@@ -29,6 +29,7 @@ import json
 import socket
 import os
 import re
+from sys import prefix
 
 import threading
 import asyncio
@@ -108,26 +109,42 @@ class RoutingServer:
     def add_res_conf(self, val: Dict[str, str]):
         if not val or not isinstance(val, dict):
             return
-        for res_k, v in val.items():
-            if res_k.startswith("/"):
-                k = res_k[1:]
-            else:
-                k = res_k
-            if k.endswith("/*"):
-                key = k[0:-1]
-            elif k.endswith("/**"):
-                key = k[0:-2]
-            elif k.endswith("/"):
-                key = k
-            else:
-                key = k + "/"
+        for k, v in val.items():
+            res_k = k
+            if not res_k.startswith("*") and res_k.endswith('/'):
+                # xxx/ equals xxx/*
+                res_k = res_k + "*"
+            if res_k.startswith('*'):
+                suffix = res_k[2:] if res_k.startswith("**") else res_k[1:]
+                assert suffix.find('/') < 0 and suffix.find('*') < 0, "If a resource path starts with *, only suffix can be configurated. "
+            if res_k.startswith('**.'):
+                # **.xxx
+                suffix = res_k[3:]
+                key = f'^[\\w%.\\-@!\\(\\)\\[\\]\\|\\$/]+\\.{suffix}$'
+            elif res_k.startswith('*.'):
+                # *.xxx
+                suffix = res_k[2:]
+                key = f'^[\\w%.\\-@!\\(\\)\\[\\]\\|\\$]+\\.{suffix}$'
+            elif res_k.endswith("/**"):
+                # xx/**
+                prefix = res_k[0:-2]
+                while prefix.startswith('/'):
+                    prefix = prefix[1:]
+                assert prefix.find("*") < 0, "You can only config a * or ** at the start or end of a path."
+                key = f'^{prefix}([\\w%.\\-@!\\(\\)\\[\\]\\|\\$/]+)$'
+            elif res_k.endswith("/*"):
+                # xx/*
+                prefix = res_k[0:-1]
+                while prefix.startswith('/'):
+                    prefix = prefix[1:]
+                assert prefix.find("*") < 0, "You can only config a * or ** at the start or end of a path."
+                key = f'^{prefix}([\\w%.\\-@!\\(\\)\\[\\]\\|\\$]+)$'
 
             if v.endswith(os.path.sep):
                 val = v
             else:
                 val = v + os.path.sep
             self._res_conf.append((key, val))
-        self._res_conf.sort(key=lambda it: -len(it[0]))
 
     def map_controller(self, ctrl: ControllerFunction):
         url = ctrl.url
@@ -148,9 +165,9 @@ class RoutingServer:
             else:
                 self.put_to_path_val_url_mapping(_method, path_pattern, ctrl, path_names)
 
-    def _res_(self, path, res_pre, res_dir):
-        fpath = os.path.join(res_dir, path.replace(res_pre, ""))
-        _logger.debug(f"static file. {path} :: {fpath}")
+    def _res_(self, fpath: str):
+        # fpath = os.path.join(res_dir, path.replace(res_regx, ""))
+        # _logger.debug(f"static file. {path} :: {fpath}")
         fext = os.path.splitext(fpath)[1]
         ext = fext.lower()
         if ext in (".html", ".htm", ".xhtml"):
@@ -200,9 +217,16 @@ class RoutingServer:
             return regexp_res
         # static files
         for k, v in self.res_conf:
-            if path.startswith(k):
+            match_static_path_conf = re.match(k, path)
+            _logger.debug(f"{path} macth static file conf {k} ? {match_static_path_conf}")
+            if match_static_path_conf:
+                if match_static_path_conf.groups():
+                    fpath = f"{v}{match_static_path_conf.group(1)}"
+                else:
+                    fpath = f"{v}{path}"
+
                 def static_fun():
-                    return self._res_(path, k, v)
+                    return self._res_(fpath)
                 return [(ControllerFunction(func=static_fun), {}, ())]
         return []
 
@@ -469,6 +493,7 @@ class CoroutineHTTPServer(RoutingServer):
         self.port: int = port
         self.ssl: SSLContext = ssl
         self.server: Server = None
+        self.__thread_local = threading.local()
 
     async def callback(self, reader: StreamReader, writer: StreamWriter):
         handler = HttpProtocolHandler(reader, writer, routing_conf=self)
@@ -488,8 +513,16 @@ class CoroutineHTTPServer(RoutingServer):
             finally:
                 await self.server.wait_closed()
 
+    def _get_event_loop(self) -> asyncio.AbstractEventLoop:
+        if not hasattr(self.__thread_local, "event_loop"):
+            try:
+                self.__thread_local.event_loop = asyncio.new_event_loop()
+            except:
+                self.__thread_local.event_loop = asyncio.get_event_loop()
+        return self.__thread_local.event_loop
+
     def start(self):
-        asyncio.run(self.start_async())
+        self._get_event_loop().run_until_complete(self.start_async())
 
     def _shutdown(self):
         _logger.debug("Try to shutdown server.")
