@@ -36,7 +36,7 @@ from uuid import uuid4
 from socket import error as SocketError
 
 from .logger import get_logger
-from simple_http_server import Headers, WebsocketCloseReason, WebsocketRequest, WebsocketSession, \
+from simple_http_server import Headers, WebsocketCloseReason, WebsocketHandlerClass, WebsocketRequest, WebsocketSession, \
     WEBSOCKET_OPCODE_BINARY, WEBSOCKET_OPCODE_CLOSE, WEBSOCKET_OPCODE_CONTINUATION, WEBSOCKET_OPCODE_PING, WEBSOCKET_OPCODE_PONG, WEBSOCKET_OPCODE_TEXT
 
 
@@ -97,7 +97,7 @@ class _ContinuationMessageCache:
 
 class _WebsocketException(Exception):
 
-    def __init__(self, graceful: bool = False,  reason: WebsocketCloseReason = None) -> None:
+    def __init__(self, reason: WebsocketCloseReason = None, graceful: bool = False) -> None:
         super().__init__(reason)
         self.__graceful: bool = graceful
         self.__reason: WebsocketCloseReason = reason
@@ -123,15 +123,15 @@ class WebsocketRequestHandler:
         self.keep_alive = True
         self.handshake_done = False
 
-        handler_class, path_values = self.routing_conf.get_websocket_handler(
-            http_protocol_handler.request_path)
-        self.handler = handler_class() if handler_class else None
+        handler_class, path_values, regroups = self.routing_conf.get_websocket_handler(http_protocol_handler.request_path)
+        self.handler = handler_class.ctrl_object if handler_class else None
         self.ws_request = WebsocketRequest()
         self.ws_request.headers = http_protocol_handler.headers
         self.ws_request.path = http_protocol_handler.request_path
         self.ws_request.query_string = http_protocol_handler.query_string
         self.ws_request.parameters = http_protocol_handler.query_parameters
         self.ws_request.path_values = path_values
+        self.ws_request.reg_groups = regroups
         if "cookie" in self.ws_request.headers:
             self.ws_request.cookies.load(self.ws_request.headers["cookie"])
         elif "Cookie" in self.ws_request.headers:
@@ -243,8 +243,7 @@ class WebsocketRequestHandler:
                 self.send_response(101, "Switching Protocols")
                 self.send_header("Upgrade", "websocket")
                 self.send_header("Connection", "Upgrade")
-                self.send_header("Sec-WebSocket-Accept",
-                                 self.calculate_response_key())
+                self.send_header("Sec-WebSocket-Accept", self.calculate_response_key())
             if headers:
                 for h_name, h_val in headers.items():
                     self.send_header(h_name, h_val)
@@ -270,7 +269,7 @@ class WebsocketRequestHandler:
     async def read_bytes(self, num):
         return await self.reader.read(num)
 
-    async def read_next_message(self):
+    async def _read_message_content(self) -> Tuple[int, int, bytearray]:
         _logger.debug(f"Read next websocket[{self.ws_request.path}] message")
         try:
             b1, b2 = await self.read_bytes(2)
@@ -304,9 +303,6 @@ class WebsocketRequestHandler:
             qb = await self.reader.read(8)
             payload_length = struct.unpack(">Q", qb)[0]
 
-        optype = OPTYPES[opcode]
-        _logger.debug(f"Receive a websocket frame: opcode[{opcode}({optype})]::fin?{bool(fin)}::payload length::{payload_length}")
-
         frame_bytes = bytearray()
         if payload_length > 0:
             masks = await self.read_bytes(4)
@@ -314,6 +310,12 @@ class WebsocketRequestHandler:
             for encoded_byte in payload:
                 frame_bytes.append(encoded_byte ^ masks[len(frame_bytes) % 4])
 
+        return fin, opcode, frame_bytes
+
+    async def read_next_message(self):
+
+        fin, opcode, frame_bytes = await self._read_message_content()
+
         if fin and opcode != WEBSOCKET_OPCODE_CONTINUATION:
             # A normal frame, handle message.
             await self.on_message(opcode, frame_bytes)
@@ -322,7 +324,7 @@ class WebsocketRequestHandler:
         if not fin and opcode != WEBSOCKET_OPCODE_CONTINUATION:
             # Fragment message: first frame, try to create a cache object.
             if opcode not in (WEBSOCKET_OPCODE_TEXT, WEBSOCKET_OPCODE_BINARY):
-                raise _WebsocketException(reason=WebsocketCloseReason(f"Control({optype}) frames MUST NOT be fragmented"))
+                raise _WebsocketException(reason=WebsocketCloseReason(f"Control({OPTYPES[opcode]}) frames MUST NOT be fragmented"))
 
             if self._continution_cache is not None:
                 # Check if another fragment message is being read.
